module PC.Json              exposing ( pcModelDecoder
                                     , pcModelEncoder
                                     , tuple2Decoder
                                     , tuple2Encoder
                                     )

import PC.Types             exposing (..)
import Maybe
import Array                exposing (Array)

import Json.Encode  as JE   exposing (Value)
import Json.Decode  as JD   exposing (Decoder)

import PC.Types             exposing (..)
import PC.UActions          exposing (..)


-- ##########################################################
-- Decoders
-- ##########################################################

pcModelDecoder : Decoder PC_Model
pcModelDecoder =
    JD.map2 
        PC_Model
        (JD.field "pc" pcDecoder)
        (JD.field "uCodes" uCodeDecoder)


pcDecoder : Decoder PC
pcDecoder =
    JD.map7
        PC
        (JD.field "ram" <| JD.array <| tuple2Decoder JD.int JD.string)
        (JD.field "dataBus" JD.int)
        (JD.field "addressBus" JD.int)
        (JD.field "instructionReg" JD.int)
        (JD.field "programmCounter" JD.int)
        (JD.field "uCounter" JD.int)
        (JD.field "accumulator" JD.int)

uCodeDecoder : Decoder (Array UAction)
uCodeDecoder =
    JD.array <| JD.map uActionDecoder JD.string

uActionDecoder : String -> UAction
uActionDecoder str =
    Maybe.withDefault UA_Nothing <| string2uAction str



tuple2Decoder : Decoder a -> Decoder b -> Decoder (a,b)
tuple2Decoder enc1 enc2 =
    JD.map2 Tuple.pair
        (JD.index 0 enc1)
        (JD.index 1 enc2)

-- ##########################################################
-- ##########################################################





-- ##########################################################
-- Encoders
-- ##########################################################

pcModelEncoder : PC_Model -> Value
pcModelEncoder model =
    JE.object
        [ ( "pc"     , pcEncoder      model.pc     )
        , ( "uCodes" , uCodesEncoder  model.uCodes )
        ]

pcEncoder : PC -> Value
pcEncoder pc =
    JE.object 
        [ ( "addressBus"        , JE.int pc.addressBus        )
        , ( "dataBus"           , JE.int pc.dataBus           )
        , ( "instructionReg"    , JE.int pc.instructionReg    )
        , ( "programmCounter"   , JE.int pc.programmCounter   )
        , ( "uCounter"          , JE.int pc.uCounter          )
        , ( "accumulator"       , JE.int pc.accumulator       )
        , ( "ram"               , ramEncoder pc.ram           )
        ]

ramEncoder : Array (Int, String) -> Value
ramEncoder arr =
    JE.array 
        (tuple2Encoder JE.int JE.string) 
        arr

uCodesEncoder : Array UAction -> Value
uCodesEncoder uActions =
    JE.array encodeSingleuCode uActions

encodeSingleuCode : UAction -> Value
encodeSingleuCode action =
    let
        uCode = uAction2UCode action
    in
    JE.string uCode.id


tuple2Encoder : (a -> Value) -> (b -> Value) -> (a,b) -> Value
tuple2Encoder encA encB (valA, valB) =
    JE.list identity [ encA valA, encB valB ]

-- ##########################################################
-- ##########################################################
-- Done.